@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2006-2020 Edgewall Software
+# Copyright (C) 2006-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -46,14 +46,6 @@ class IAdminPanelProvider(Interface):
         This function should return a tuple of the form `(template, data)`,
         where `template` is the name of the template to use and `data` is
         the data to use when rendering the template.
-
-        .. note::
-
-           When a plugin wants to use a legacy Genshi template instead
-           of a Jinja2 template, it needs to return instead a *triple*
-           of the form ``(template, data, None)``, similar to what
-           `.IRequestHandler.process_request` does.
-
         """
 
 
@@ -229,7 +221,7 @@ def get_console_locale(env=None, lang=None,
             locales.append(value)
         if env:
             locales.append(env.config.get('trac', 'default_language'))
-        locales = filter(None, map(normalize, locales))
+        locales = list(filter(None, map(normalize, locales)))
         try:
             return get_negotiated_locale(locales)
         except UnknownLocaleError:
