@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2007 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -23,10 +23,10 @@ import setuptools
 import shutil
 import sys
 import time
-from ConfigParser import RawConfigParser
-from subprocess import PIPE
+from configparser import RawConfigParser
+from subprocess import PIPE, Popen
 from tempfile import mkdtemp
-from urlparse import urlsplit
+from urllib.parse import urlsplit
 
 from trac import log
 from trac.admin.api import (AdminCommandError, IAdminCommandProvider,
@@ -43,7 +43,7 @@ from trac.db.convert import copy_tables
 from trac.loader import load_components
 from trac.util import as_bool, backup_config_file, copytree, create_file, \
                       get_pkginfo, is_path_below, lazy, makedirs
-from trac.util.compat import Popen, close_fds
+from trac.util.compat import close_fds
 from trac.util.concurrency import threading
 from trac.util.datefmt import pytz
 from trac.util.text import exception_to_unicode, path_to_unicode, printerr, \
@@ -85,36 +85,34 @@ class Environment(Component, ComponentManager):
     setup_participants = ExtensionPoint(IEnvironmentSetupParticipant)
 
     components_section = ConfigSection('components',
-        """This section is used to enable or disable components
-        provided by plugins, as well as by Trac itself. The component
-        to enable/disable is specified via the name of the
-        option. Whether its enabled is determined by the option value;
-        setting the value to `enabled` or `on` will enable the
-        component, any other value (typically `disabled` or `off`)
-        will disable the component.
+        """Enable or disable components provided by Trac and plugins.
+        The component to enable/disable is specified by the option name.
+        The enabled state is determined by the option value: setting
+        the value to `enabled` or `on` will enable the component, any
+        other value (typically `disabled` or `off`) will disable the
+        component.
 
         The option name is either the fully qualified name of the
-        components or the module/package prefix of the component. The
+        component or the module/package prefix of the component. The
         former enables/disables a specific component, while the latter
-        enables/disables any component in the specified
-        package/module.
+        enables/disables any component in the specified package/module.
 
         Consider the following configuration snippet:
-        {{{
+        {{{#!ini
         [components]
         trac.ticket.report.ReportModule = disabled
         acct_mgr.* = enabled
         }}}
 
         The first option tells Trac to disable the
-        [wiki:TracReports report module].
+        [TracReports report module].
         The second option instructs Trac to enable all components in
-        the `acct_mgr` package. Note that the trailing wildcard is
-        required for module/package matching.
+        the `acct_mgr` package. The trailing wildcard is required for
+        module/package matching.
 
         To view the list of active components, go to the ''Plugins''
-        page on ''About Trac'' (requires `CONFIG_VIEW`
-        [wiki:TracPermissions permissions]).
+        section of ''About Trac'' (requires `CONFIG_VIEW`
+        [TracPermissions permission]).
 
         See also: TracPlugins
         """)
@@ -131,12 +129,12 @@ class Environment(Component, ComponentManager):
         """)
 
     base_url = Option('trac', 'base_url', '',
-        """Reference URL for the Trac deployment.
+        """Base URL of the Trac site.
 
-        This is the base URL that will be used when producing
-        documents that will be used outside of the web browsing
-        context, like for example when inserting URLs pointing to Trac
-        resources in notification e-mails.""")
+        This is used to produce documents outside of the web browsing
+        context, such as URLs in notification e-mails that point to
+        Trac resources.
+        """)
 
     base_url_for_redirect = BoolOption('trac', 'use_base_url_for_redirect',
                                         False,
@@ -173,9 +171,14 @@ class Environment(Component, ComponentManager):
         """Short description of the project.""")
 
     project_url = Option('project', 'url', '',
-        """URL of the main project web site, usually the website in
-        which the `base_url` resides. This is used in notification
-        e-mails.""")
+        """URL of the project web site.
+
+        This is usually the domain in which the `base_url` resides.
+        For example, the project URL might be !https://myproject.com,
+        with the Trac site (`base_url`) residing at either
+        !https://trac.myproject.com or !https://myproject.com/trac.
+        The project URL is added to the footer of notification e-mails.
+        """)
 
     project_admin = Option('project', 'admin', '',
         """E-Mail address of the project's administrator.""")
@@ -239,15 +242,16 @@ class Environment(Component, ComponentManager):
         `($(thread)d) Trac[$(basename)s:$(module)s] $(levelname)s: $(message)s`
         """)
 
-    def __init__(self, path, create=False, options=[]):
+    def __init__(self, path, create=False, options=[], default_data=True):
         """Initialize the Trac environment.
 
         :param path:   the absolute path to the Trac environment
-        :param create: if `True`, the environment is created and
-                       populated with default data; otherwise, the
-                       environment is expected to already exist.
+        :param create: if `True`, the environment is created and otherwise,
+                       the environment is expected to already exist.
         :param options: A list of `(section, name, value)` tuples that
                         define configuration options
+        :param default_data: if `True` (the default), the environment is
+                             populated with default data when created.
         """
         ComponentManager.__init__(self)
 
@@ -256,7 +260,7 @@ class Environment(Component, ComponentManager):
         self.config = None
 
         if create:
-            self.create(options)
+            self.create(options, default_data)
             for setup_participant in self.setup_participants:
                 setup_participant.environment_created()
         else:
@@ -293,15 +297,6 @@ class Environment(Component, ComponentManager):
         return sorted(set(info),
                       key=lambda args: (args[0] != 'Trac', args[0].lower()))
 
-    def get_systeminfo(self):
-        """Return a list of `(name, version)` tuples describing the name
-        and version information of external packages used by Trac and plugins.
-
-        :since 1.3.1: deprecated and will be removed in 1.5.1. Use
-                      system_info property instead.
-        """
-        return self.system_info
-
     # ISystemInfoProvider methods
 
     def get_system_info(self):
@@ -326,7 +321,7 @@ class Environment(Component, ComponentManager):
 
     def _component_name(self, name_or_class):
         name = name_or_class
-        if not isinstance(name_or_class, basestring):
+        if not isinstance(name_or_class, str):
             name = name_or_class.__module__ + '.' + name_or_class.__name__
         return name.lower()
 
@@ -370,7 +365,7 @@ class Environment(Component, ComponentManager):
     def enable_component(self, cls):
         """Enable a component or module."""
         self._component_rules[self._component_name(cls)] = True
-        super(Environment, self).enable_component(cls)
+        super().enable_component(cls)
 
     @contextmanager
     def component_guard(self, component, reraise=False):
@@ -400,8 +395,9 @@ class Environment(Component, ComponentManager):
         """Verify that the provided path points to a valid Trac environment
         directory."""
         try:
-            with open(os.path.join(self.path, 'VERSION')) as f:
-                tag = f.readline().rstrip('\n')
+            with open(os.path.join(self.path, 'VERSION'),
+                      encoding='utf-8') as f:
+                tag = f.readline().rstrip()
         except Exception as e:
             raise TracError(_("No Trac environment found at %(path)s\n"
                               "%(e)s",
@@ -511,7 +507,7 @@ class Environment(Component, ComponentManager):
         if tid is None:
             log.shutdown(self.log)
 
-    def create(self, options=[]):
+    def create(self, options=[], default_data=True):
         """Create the basic directory structure of the environment,
         initialize the database and populate the configuration file
         with default values.
@@ -563,7 +559,10 @@ class Environment(Component, ComponentManager):
         self._update_sample_config()
 
         # Create the database
-        DatabaseManager(self).init_db()
+        dbm = DatabaseManager(self)
+        dbm.init_db()
+        if default_data:
+            dbm.insert_default_data()
 
     @lazy
     def database_version(self):
@@ -621,7 +620,7 @@ class Environment(Component, ComponentManager):
         path = self.path
         for dir in dirs:
             path = os.path.join(path, dir)
-        return os.path.realpath(path)
+        return os.path.normcase(os.path.realpath(path))
 
     @lazy
     def attachments_dir(self):
@@ -689,7 +688,8 @@ class Environment(Component, ComponentManager):
                       self.trac_version)
 
     def create_logger(self, log_type, log_file, log_level, log_format):
-        log_id = 'Trac.%s' % hashlib.sha1(self.path).hexdigest()
+        log_id = 'Trac.%s' % \
+                 hashlib.sha1(self.path.encode('utf-8')).hexdigest()
         if log_format:
             log_format = log_format.replace('$(', '%(') \
                                    .replace('%(path)s', self.path) \
@@ -779,7 +779,7 @@ class Environment(Component, ComponentManager):
             try:
                 self.backup(backup_dest)
             except Exception as e:
-                raise BackupError(e)
+                raise BackupError(e) from e
 
         for participant in upgraders:
             self.log.info("upgrading %s...", participant)
@@ -995,8 +995,8 @@ class EnvironmentAdmin(Component):
             template = chrome.load_template('deploy_trac.' + script, text=True)
             text = chrome.render_template_string(template, data, text=True)
 
-            with open(dest, 'w') as out:
-                out.write(text.encode('utf-8'))
+            with open(dest, 'w', encoding='utf-8') as out:
+                out.write(text)
 
     def _do_hotcopy(self, dest, no_db=None):
         if no_db not in (None, '--no-database'):
@@ -1129,13 +1129,13 @@ class EnvironmentAdmin(Component):
 
     def _create_env(self, env_path, dburi):
         parser = RawConfigParser()
-        parser.read(self.env.config_file_path)
+        parser.read(self.env.config_file_path, 'utf-8')
         options = dict(((section, name), value)
                        for section in parser.sections()
                        for name, value in parser.items(section))
         options[('trac', 'database')] = dburi
         options = sorted((section, name, value) for (section, name), value
-                                                in options.iteritems())
+                                                in options.items())
 
         class MigrateEnvironment(Environment):
             abstract = True
