@@ -49,7 +49,8 @@
         var expander = selection.find('.expander');
         if (expander.length > 0) {
           if (event.keyCode == 82) { // 'r'eload
-            selection.removeClass("expanded").removeClass("collapsed")
+            expander.removeClass("expanded");
+            selection.removeClass("collapsed")
               .siblings("tr."+selection.get(0).id).not(selection).remove();
           }
           expander.click();
@@ -85,11 +86,11 @@
   });
 
   $(function() {
-    $('a,input,select,textarea,button').bind({
+    $('a,input,select,textarea,button').on({
       focus: function() { ENABLE_KEY_NAV = false; },
       blur: function() { ENABLE_KEY_NAV = true; }
     });
-    $("#dirlist tr").live('mouseenter', function() {
+    $(document).on("mouseenter", "#dirlist tr", function() {
       LAST_HOVERED_FILE_ELEM = $(this);
     });
   });
