@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -21,36 +21,11 @@
 import io
 import re
 import sys
-from HTMLParser import HTMLParser
-import htmlentitydefs as entities
+from html import entities
+from html.parser import HTMLParser
 
 from markupsafe import Markup, escape as escape_quotes
 
-"""Utilities for producing HTML content.
-
-Imports related to the legacy Genshi template engine should all go
-through this module:
-
-    from trac.util.html import genshi, Stream
-
-If Genshi is not installed, `genshi` and all related symbols will be
-`None`.
-
-"""
-
-try:
-    import genshi
-    import genshi.input
-    from genshi.core import Attrs, QName, Stream, COMMENT, START, END, TEXT
-    from genshi.input import ParseError
-    def stream_to_unicode(stream):
-        return Markup(stream.render('xhtml', encoding=None,
-                                    strip_whitespace=False))
-except ImportError:
-    genshi = stream_to_unicode = None
-    HTML = COMMENT = START = END = TEXT = Attrs = QName = Stream = None
-    ParseError = None
-
 try:
     from babel.support import LazyProxy
 except ImportError:
@@ -69,7 +44,7 @@ _name2codepoint = entities.name2codepoint.copy()
 _name2codepoint['apos'] = 39  # single quote
 
 
-def escape(str, quotes=True):
+def escape(text, quotes=True):
     """Create a Markup instance from a string and escape special characters
     it may contain (<, >, & and \").
 
@@ -79,60 +54,60 @@ def escape(str, quotes=True):
                    addition to the other special characters
 
     >>> escape('"1 < 2"')
-    Markup(u'&#34;1 &lt; 2&#34;')
+    Markup('&#34;1 &lt; 2&#34;')
 
     >>> escape(['"1 < 2"'])
-    Markup(u"['&#34;1 &lt; 2&#34;']")
+    Markup("['&#34;1 &lt; 2&#34;']")
 
     If the `quotes` parameter is set to `False`, the \" character is left
     as is. Escaping quotes is generally only required for strings that are
     to be used in attribute values.
 
     >>> escape('"1 < 2"', quotes=False)
-    Markup(u'"1 &lt; 2"')
+    Markup('"1 &lt; 2"')
 
     >>> escape(['"1 < 2"'], quotes=False)
-    Markup(u'[\\'"1 &lt; 2"\\']')
+    Markup('[\\'"1 &lt; 2"\\']')
 
     However, `escape` behaves slightly differently with `Markup` and
     `Fragment` behave instances, as they are passed through
     unmodified.
 
     >>> escape(Markup('"1 < 2 &#39;"'))
-    Markup(u'"1 < 2 &#39;"')
+    Markup('"1 < 2 &#39;"')
 
     >>> escape(Markup('"1 < 2 &#39;"'), quotes=False)
-    Markup(u'"1 < 2 &#39;"')
+    Markup('"1 < 2 &#39;"')
 
     >>> escape(tag.b('"1 < 2"'))
-    Markup(u'<b>"1 &lt; 2"</b>')
+    Markup('<b>"1 &lt; 2"</b>')
 
     >>> escape(tag.b('"1 < 2"'), quotes=False)
-    Markup(u'<b>"1 &lt; 2"</b>')
+    Markup('<b>"1 &lt; 2"</b>')
 
     :return: the escaped `Markup` string
     :rtype: `Markup`
 
     """
-    if isinstance(str, Markup):
-        return str
-    if isinstance(str, Fragment):
-        return Markup(str)
-    e = escape_quotes(str)
+    if isinstance(text, Markup):
+        return text
+    if isinstance(text, Fragment):
+        return Markup(text)
+    e = escape_quotes(text)
     if quotes:
         if '&#39;' not in e:
             return e
-        return Markup(unicode(e).replace('&#39;', "'"))
+        return Markup(str(e).replace('&#39;', "'"))
     elif '&#3' not in e:
         return e
-    return Markup(unicode(e).replace('&#34;', '"').replace('&#39;', "'"))
+    return Markup(str(e).replace('&#34;', '"').replace('&#39;', "'"))
 
 
 def unescape(text):
-    """Reverse-escapes &, <, >, and \" and returns a `unicode` object.
+    """Reverse-escapes &, <, >, and \" and returns a `str` object.
 
     >>> unescape(Markup('1 &lt; 2'))
-    u'1 < 2'
+    '1 < 2'
 
     If the provided `text` object is not a `Markup` instance, it is returned
     unchanged.
@@ -142,7 +117,7 @@ def unescape(text):
 
     :param text: the text to unescape
     :return: the unescsaped string
-    :rtype: `unicode`
+    :rtype: `str`
     """
     if not text:
         return ''
@@ -157,24 +132,24 @@ def stripentities(text, keepxmlentities=False):
     replaced by the equivalent UTF-8 characters.
 
     >>> stripentities('1 &lt; 2')
-    u'1 < 2'
+    '1 < 2'
     >>> stripentities('more &hellip;')
-    u'more \u2026'
+    'more \u2026'
     >>> stripentities('&#8230;')
-    u'\u2026'
+    '\u2026'
     >>> stripentities('&#x2026;')
-    u'\u2026'
-    >>> stripentities(Markup(u'\u2026'))
-    u'\u2026'
+    '\u2026'
+    >>> stripentities(Markup('\u2026'))
+    '\u2026'
 
     If the `keepxmlentities` parameter is provided and is a truth value, the
     core XML entities (&amp;, &apos;, &gt;, &lt; and &quot;) are left intact.
 
     >>> stripentities('1 &lt; 2 &hellip;', keepxmlentities=True)
-    u'1 &lt; 2 \u2026'
+    '1 &lt; 2 \u2026'
 
-    :return: a `unicode` instance with entities removed
-    :rtype: `unicode`
+    :return: a `str` instance with entities removed
+    :rtype: `str`
     """
     def _replace_entity(match):
         if match.group(1): # numeric entity
@@ -183,20 +158,20 @@ def stripentities(text, keepxmlentities=False):
                 ref = int(ref[1:], 16)
             else:
                 ref = int(ref, 10)
-            return _unichr(ref)
+            return chr(ref)
         else: # character entity
             ref = match.group(2)
             if keepxmlentities and ref in ('amp', 'apos', 'gt', 'lt', 'quot'):
                 return '&%s;' % ref
             try:
-                return _unichr(_name2codepoint[ref])
+                return chr(_name2codepoint[ref])
             except KeyError:
                 if keepxmlentities:
                     return '&amp;%s;' % ref
                 else:
                     return ref
     if isinstance(text, Markup):
-        text = unicode(text)
+        text = str(text)
     return _STRIPENTITIES_RE.sub(_replace_entity, text)
 
 
@@ -204,20 +179,20 @@ def striptags(text):
     """Return a copy of the text with any XML/HTML tags removed.
 
     >>> striptags('<span>Foo</span> bar')
-    u'Foo bar'
+    'Foo bar'
     >>> striptags('<span class="bar">Foo</span>')
-    u'Foo'
+    'Foo'
     >>> striptags('Foo<br />')
-    u'Foo'
+    'Foo'
 
     HTML/XML comments are stripped, too:
 
     >>> striptags('<!-- <blub>hehe</blah> -->test')
-    u'test'
+    'test'
 
     :param text: the string to remove tags from
-    :return: a `unicode` instance with all tags removed
-    :rtype: `unicode`
+    :return: a `str` instance with all tags removed
+    :rtype: `str`
     """
     return Markup(text).striptags()
 
@@ -286,25 +261,28 @@ def classes(*args, **kwargs):
     positional arguments must be strings:
 
     >>> classes('foo', 'bar')
-    u'foo bar'
+    'foo bar'
 
     In addition, the names of any supplied keyword arguments are added
     if they have a truth value:
 
     >>> classes('foo', bar=True)
-    u'foo bar'
+    'foo bar'
     >>> classes('foo', bar=False)
-    u'foo'
+    'foo'
+    >>> classes(foo=True, bar=True)
+    'bar foo'
 
     If none of the arguments are added to the list, this function
     returns `''`:
 
     >>> classes(bar=False)
-    u''
+    ''
 
     """
-    classes = list(filter(None, args)) + [k for k, v in kwargs.items() if v]
-    return u' '.join(classes)
+    classes = list(filter(None, args))
+    classes.extend(k for k in sorted(kwargs) if kwargs[k])
+    return ' '.join(classes)
 
 def styles(*args, **kwargs):
     """Helper function for dynamically assembling a list of CSS style name
@@ -314,34 +292,35 @@ def styles(*args, **kwargs):
     positional arguments must be strings or dicts:
 
     >>> styles('foo: bar', 'fu: baz', {'bottom-right': '1em'})
-    u'foo: bar; fu: baz; bottom-right: 1em'
+    'foo: bar; fu: baz; bottom-right: 1em'
 
     In addition, the names of any supplied keyword arguments are added
     if they have a string value:
 
-    >>> styles(foo='bar', fu='baz')
-    u'foo: bar; fu: baz'
-    >>> styles(foo='bar', bar=False)
-    u'foo: bar'
+    >>> styles('foo: bar', fu='baz')
+    'foo: bar; fu: baz'
+    >>> styles('foo: bar', bar=False)
+    'foo: bar'
 
     If none of the arguments are added to the list, this function
     returns `''`:
 
     >>> styles(bar=False)
-    u''
+    ''
 
     """
-    args = list(filter(None, args))
     d = {}
     styles = []
-    for arg in args:
+    for arg in filter(None, args):
         if isinstance(arg, dict):
             d.update(arg)
         else:
             styles.append(arg)
     d.update(kwargs)
-    styles.extend('%s: %s' % (k, v) for k, v in d.iteritems() if v)
-    return u'; '.join(styles)
+    styles.extend('%s: %s' % (k, v)
+                  for k, v in sorted(d.items(), key=lambda i: i[0])
+                  if v)
+    return '; '.join(styles)
 
 
 class Fragment(object):
@@ -355,25 +334,23 @@ class Fragment(object):
             self.append(arg)
 
     def __html__(self):
-        return Markup(unicode(self))
-
-    def __unicode__(self):
-        return u''.join(escape(c, False) for c in self.children)
+        return Markup(str(self))
 
     def __str__(self):
-        return self.__unicode__().encode('utf-8')
+        return ''.join(escape(c, False) for c in self.children)
 
     def __add__(self, other):
         return Fragment(self, other)
 
+    def __call__(self, *args):
+        for arg in args:
+            self.append(arg)
+        return self
+
     def append(self, arg):
-        global genshi
         if arg: # ignore most false values (None, False, [], (), ''), except 0!
-            if isinstance(arg, (Fragment, basestring, int, float, long)):
+            if isinstance(arg, (Fragment, str, bytes, int, float)):
                 self.children.append(arg)
-            elif genshi and isinstance(arg, Stream):
-                # legacy support for Genshi streams
-                self.children.append(stream_to_unicode(arg))
             else:
                 # support iterators and generators
                 try:
@@ -382,20 +359,12 @@ class Fragment(object):
                 except TypeError:
                     self.children.append(arg)
         elif arg == 0:
-            self.children.append(u'0')
+            self.children.append('0')
 
     def as_text(self):
-        return u''.join(c.as_text() if isinstance(c, Fragment) else unicode(c)
+        return ''.join(c.as_text() if isinstance(c, Fragment) else str(c)
                         for c in self.children)
 
-    if genshi:
-        def __iter__(self):
-            """Genshi compatibility layer.
-
-            :deprecated: this will be removed in Trac 1.5.1.
-            """
-            yield TEXT, Markup(self), (None, -1, -1)
-
 
 class XMLElement(Fragment):
     """An element represents an XML element, with a tag name, attributes
@@ -409,11 +378,11 @@ class XMLElement(Fragment):
 
     VOID_ELEMENTS = ()
 
-    CLOSE_TAG = u'/>'
+    CLOSE_TAG = '/>'
 
     def __init__(self, tag, *args, **kwargs):
         Fragment.__init__(self, *args)
-        self.tag = unicode(tag)
+        self.tag = str(tag)
         self.attrib = self._dict_from_kwargs(kwargs) \
                       if kwargs else self.EMPTY_ATTRIB
 
@@ -422,7 +391,7 @@ class XMLElement(Fragment):
 
     def _dict_from_kwargs(self, kwargs):
         attrs = []
-        for k, v in kwargs.iteritems():
+        for k, v in kwargs.items():
             if v is not None:
                 if k[-1:] == '_':
                     k = k[:-1]
@@ -443,8 +412,8 @@ class XMLElement(Fragment):
             self.append(arg)
         return self
 
-    def __unicode__(self):
-        elt = u'<' + self.tag
+    def __str__(self):
+        elt = '<' + self.tag
         if self.attrib:
             # Sorting the attributes makes the unit-tests more robust
             attrs = []
@@ -453,10 +422,10 @@ class XMLElement(Fragment):
                 if v:
                     attrs.append(' %s="%s"' % (k, v))
             if attrs:
-                elt += u''.join(attrs)
+                elt += ''.join(attrs)
         if self.children or (self.VOID_ELEMENTS and
                              self.tag not in self.VOID_ELEMENTS):
-            elt += u'>' + Fragment.__unicode__(self) + u'</' + self.tag + u'>'
+            elt += '>' + Fragment.__str__(self) + '</' + self.tag + '>'
         else:
             elt += self.CLOSE_TAG
         return elt
@@ -474,7 +443,7 @@ class Element(XMLElement):
     VOID_ELEMENTS = {'area', 'base', 'br', 'col', 'command', 'embed', 'hr',
                      'img', 'input', 'keygen', 'link', 'meta', 'param',
                      'source', 'track', 'wbr'}
-    CLOSE_TAG = u' />'
+    CLOSE_TAG = ' />'
 
     __slots__ = ()
 
@@ -516,9 +485,6 @@ class TracHTMLSanitizer(object):
     The usual way to use the sanitizer is to call the `sanitize`
     method on some potentially unsafe HTML content.
 
-    Note that for backward compatibility, the TracHTMLSanitizer still
-    behaves as a Genshi filter.
-
     See also `genshi.HTMLSanitizer`_ from which the TracHTMLSanitizer
     has evolved.
 
@@ -611,25 +577,25 @@ class TracHTMLSanitizer(object):
 
     # IE6 <http://heideri.ch/jso/#80>
     _EXPRESSION_SEARCH = re.compile(
-        u'[eE\uFF25\uFF45]'         # FULLWIDTH LATIN CAPITAL LETTER E
+        '[eE\uFF25\uFF45]'         # FULLWIDTH LATIN CAPITAL LETTER E
                                     # FULLWIDTH LATIN SMALL LETTER E
-        u'[xX\uFF38\uFF58]'         # FULLWIDTH LATIN CAPITAL LETTER X
+        '[xX\uFF38\uFF58]'         # FULLWIDTH LATIN CAPITAL LETTER X
                                     # FULLWIDTH LATIN SMALL LETTER X
-        u'[pP\uFF30\uFF50]'         # FULLWIDTH LATIN CAPITAL LETTER P
+        '[pP\uFF30\uFF50]'         # FULLWIDTH LATIN CAPITAL LETTER P
                                     # FULLWIDTH LATIN SMALL LETTER P
-        u'[rR\u0280\uFF32\uFF52]'   # LATIN LETTER SMALL CAPITAL R
+        '[rR\u0280\uFF32\uFF52]'   # LATIN LETTER SMALL CAPITAL R
                                     # FULLWIDTH LATIN CAPITAL LETTER R
                                     # FULLWIDTH LATIN SMALL LETTER R
-        u'[eE\uFF25\uFF45]'         # FULLWIDTH LATIN CAPITAL LETTER E
+        '[eE\uFF25\uFF45]'         # FULLWIDTH LATIN CAPITAL LETTER E
                                     # FULLWIDTH LATIN SMALL LETTER E
-        u'[sS\uFF33\uFF53]{2}'      # FULLWIDTH LATIN CAPITAL LETTER S
+        '[sS\uFF33\uFF53]{2}'      # FULLWIDTH LATIN CAPITAL LETTER S
                                     # FULLWIDTH LATIN SMALL LETTER S
-        u'[iI\u026A\uFF29\uFF49]'   # LATIN LETTER SMALL CAPITAL I
+        '[iI\u026A\uFF29\uFF49]'   # LATIN LETTER SMALL CAPITAL I
                                     # FULLWIDTH LATIN CAPITAL LETTER I
                                     # FULLWIDTH LATIN SMALL LETTER I
-        u'[oO\uFF2F\uFF4F]'         # FULLWIDTH LATIN CAPITAL LETTER O
+        '[oO\uFF2F\uFF4F]'         # FULLWIDTH LATIN CAPITAL LETTER O
                                     # FULLWIDTH LATIN SMALL LETTER O
-        u'[nN\u0274\uFF2E\uFF4E]'   # LATIN LETTER SMALL CAPITAL N
+        '[nN\u0274\uFF2E\uFF4E]'   # LATIN LETTER SMALL CAPITAL N
                                     # FULLWIDTH LATIN CAPITAL LETTER N
                                     # FULLWIDTH LATIN SMALL LETTER N
     ).search
@@ -637,14 +603,14 @@ class TracHTMLSanitizer(object):
     # IE6 <http://openmya.hacker.jp/hasegawa/security/expression.txt>
     #     7) Particular bit of Unicode characters
     _URL_FINDITER = re.compile(
-        u'[Uu][Rr\u0280][Ll\u029F]\s*\(([^)]+)').finditer
+        '[Uu][Rr\u0280][Ll\u029F]\s*\(([^)]+)').finditer
 
     def sanitize(self, html):
         """Transforms the incoming HTML by removing anything's that deemed
         unsafe.
 
         :param html: the input HTML
-        :type: basestring
+        :type: str
         :return: the sanitized content
         :rtype: Markup
 
@@ -654,40 +620,6 @@ class TracHTMLSanitizer(object):
         transform.close()
         return Markup(transform.out.getvalue())
 
-    if genshi:
-        def __call__(self, stream):
-            """Apply the filter to the given stream.
-
-            :deprecated: the ability to behave as a Genshi filter will be
-                         removed in Trac 1.5.1.
-
-            :param stream: the markup event stream to filter
-            """
-            waiting_for = None
-
-            for kind, data, pos in stream:
-                if kind is START:
-                    if waiting_for:
-                        continue
-                    tag, attrs = data
-                    if not self.is_safe_elem(tag, attrs):
-                        waiting_for = tag
-                        continue
-                    new_attrs = self.sanitize_attrs(tag, dict(attrs))
-                    yield kind, (tag, Attrs(new_attrs.iteritems())), pos
-
-                elif kind is END:
-                    tag = data
-                    if waiting_for:
-                        if waiting_for == tag:
-                            waiting_for = None
-                    else:
-                        yield kind, data, pos
-
-                elif kind is not COMMENT:
-                    if not waiting_for:
-                        yield kind, data, pos
-
     def is_safe_css(self, prop, value):
         """Determine whether the given css property declaration is to be
         considered safe for inclusion in the output.
@@ -708,26 +640,17 @@ class TracHTMLSanitizer(object):
         inclusion in the output.
 
         :param tag: the tag name of the element
-        :type tag: QName or basestring
+        :type tag: str
         :param attrs: the element attributes
-        :type attrs: Attrs or list
+        :type attrs: list
         :return: whether the element should be considered safe
         :rtype: bool
 
         """
         if tag not in self.safe_tags:
             return False
-        if hasattr(tag, 'localname'): # in Genshi QName
-            tag = tag.localname
-        if tag == 'input':
-            # TODO (1.5.1) no more Attrs
-            if Attrs and isinstance(attrs, Attrs):
-                input_type = attrs.get('type', '').lower()
-                if input_type == 'password':
-                    return False
-            else:
-                if ('type', 'password') in attrs:
-                    return False
+        if tag == 'input' and ('type', 'password') in attrs:
+            return False
         return True
 
     def is_safe_uri(self, uri):
@@ -766,7 +689,7 @@ class TracHTMLSanitizer(object):
 
         """
         new_attrs = {}
-        for attr, value in attrs.iteritems():
+        for attr, value in attrs.items():
             if value is None:
                 value = attr
             if attr not in self.safe_attrs:
@@ -785,8 +708,6 @@ class TracHTMLSanitizer(object):
         if tag == 'img' and 'src' in new_attrs and \
                 not self._is_safe_origin(new_attrs['src']):
             attr = 'crossorigin'
-            if QName and isinstance(tag, QName):
-                attr = QName(attr)
             new_attrs[attr] = 'anonymous'
         return new_attrs
 
@@ -797,23 +718,23 @@ class TracHTMLSanitizer(object):
         with a scheme that is not considered safe are removed:
 
         >>> sanitizer = TracHTMLSanitizer()
-        >>> sanitizer.sanitize_css(u'''
+        >>> sanitizer.sanitize_css('''
         ...   background: url(javascript:alert("foo"));
         ...   color: #000;
         ... ''')
-        [u'color: #000']
+        ['color: #000']
 
         Also, the proprietary Internet Explorer function
         ``expression()`` is always stripped:
 
-        >>> sanitizer.sanitize_css(u'''
+        >>> sanitizer.sanitize_css('''
         ...   background: #fff;
         ...   color: #000;
         ...   width: e/**/xpression(alert("F"));
         ... ''')
-        [u'background: #fff', u'color: #000', u'width: e xpression(alert("F"))']
+        ['background: #fff', 'color: #000', 'width: e xpression(alert("F"))']
 
-        :param text: the CSS text; this is expected to be `unicode` and to not
+        :param text: the CSS text; this is expected to be `str` and to not
                      contain any character or numeric references
         :return: a list of declarations that are considered safe
         :rtype: `list`
@@ -851,14 +772,14 @@ class TracHTMLSanitizer(object):
             t = match.group(1)
             if t:
                 code = int(t, 16)
-                chr = _unichr(code)
+                c = chr(code)
                 if code <= 0x1f:
                     # replace space character because IE ignores control
                     # characters
-                    chr = ' '
-                elif chr == '\\':
-                    chr = r'\\'
-                return chr
+                    c = ' '
+                elif c == '\\':
+                    c = r'\\'
+                return c
             t = match.group(2)
             if t == '\\':
                 return r'\\'
@@ -925,7 +846,7 @@ class HTMLTransform(HTMLParser):
                                       if isinstance(v, bytes) else v
         elif isinstance(out, io.IOBase):
             self._convert = lambda v: v.encode('utf-8') \
-                                      if isinstance(v, unicode) else v
+                                      if isinstance(v, str) else v
         else:
             self._convert = lambda v: v
 
@@ -935,12 +856,6 @@ class HTMLTransform(HTMLParser):
     def handle_startendtag(self, tag, attrs):
         self._write(self.get_starttag_text())
 
-    def handle_charref(self, name):
-        self._handle_charref(name)
-
-    def handle_entityref(self, name):
-        self._handle_entityref(name)
-
     def handle_comment(self, data):
         self._write('<!--%s-->' % data)
 
@@ -956,31 +871,6 @@ class HTMLTransform(HTMLParser):
     def handle_endtag(self, tag):
         self._write('</' + tag + '>')
 
-    def unescape(self, s):
-        return _html_parser_unescape(s)
-
-    _codepoint2ref = {38: '&amp;', 60: '&lt;', 62: '&gt;', 34: '&#34;'}
-
-    def _handle_charref(self, name):
-        if name.startswith(('x', 'X')):
-            codepoint = int(name[1:], 16)
-        else:
-            codepoint = int(name)
-        if 0 <= codepoint <= 0x10ffff:
-            text = self._codepoint2ref.get(codepoint) or _unichr(codepoint)
-        else:
-            text = '&amp;#%s;' % name
-        self._write(text)
-
-    def _handle_entityref(self, name):
-        try:
-            codepoint = _name2codepoint[name]
-        except KeyError:
-            text = '&amp;%s;' % name
-        else:
-            text = self._codepoint2ref.get(codepoint) or _unichr(codepoint)
-        self._write(text)
-
     def _write(self, data):
         self.out.write(self._convert(data))
 
@@ -1021,8 +911,8 @@ class HTMLSanitization(HTMLTransform):
             return
 
         new_attrs = self.sanitizer.sanitize_attrs(tag, dict(attrs))
-        html_attrs = ''.join(' %s="%s"' % (name, escape(value))
-                             for name, value in new_attrs.iteritems())
+        html_attrs = ''.join(' %s="%s"' % (name, escape(new_attrs[name]))
+                             for name in sorted(new_attrs))
         self._write('<%s%s%s>' % (tag, html_attrs, startend))
 
     def handle_starttag(self, tag, attrs):
@@ -1033,14 +923,6 @@ class HTMLSanitization(HTMLTransform):
         if not self.waiting_for:
             self._handle_start(tag, attrs, '/')
 
-    def handle_charref(self, name):
-        if not self.waiting_for:
-            self._handle_charref(name)
-
-    def handle_entityref(self, name):
-        if not self.waiting_for:
-            self._handle_entityref(name)
-
     def handle_comment(self, data):
         pass
 
@@ -1068,15 +950,15 @@ def plaintext(text, keeplinebreaks=True):
     """Extract the text elements from (X)HTML content
 
     >>> plaintext('<b>1 &lt; 2</b>')
-    u'1 < 2'
+    '1 < 2'
 
     >>> plaintext(tag('1 ', tag.b('<'), ' 2'))
-    u'1 < 2'
+    '1 < 2'
 
     >>> plaintext('''<b>1
     ... &lt;
     ... 2</b>''', keeplinebreaks=False)
-    u'1 < 2'
+    '1 < 2'
 
     :param text: `unicode` or `Fragment`
     :param keeplinebreaks: optionally keep linebreaks
@@ -1087,7 +969,7 @@ def plaintext(text, keeplinebreaks=True):
     else:
         text = stripentities(striptags(text))
     if not keeplinebreaks:
-        text = text.replace(u'\n', u' ')
+        text = text.replace('\n', ' ')
     return text
 
 
@@ -1152,123 +1034,17 @@ def to_fragment(input):
 
 
 # Mappings for removal of control characters
-_translate_nop = ''.join(chr(i) for i in xrange(256))
-_invalid_control_chars = ''.join(chr(i) for i in xrange(32)
-                                 if i not in [0x09, 0x0a, 0x0d])
+_invalid_control_chars = bytes(i for i in range(32)
+                                 if i not in (0x09, 0x0a, 0x0d))
 
-def valid_html_bytes(bytes):
-    return bytes.translate(_translate_nop, _invalid_control_chars)
+def valid_html_bytes(data):
+    """Return only valid bytes in XML/HTML from the given data.
 
+    >>> valid_html_bytes(b'blah')
+    b'blah'
 
-if sys.maxunicode > 0xffff:
-    _unichr = unichr
-else:
-    def _unichr(codepoint):  # narrow Python build
-        try:
-            return unichr(codepoint)
-        except ValueError:
-            if not (0 <= codepoint <= 0x10ffff):
-                raise
-            s = r'\U%08x' % codepoint
-            try:
-                return s.decode('unicode-escape')
-            except Exception as e:
-                raise ValueError(e)
+    >>> list(valid_html_bytes(bytes(range(33)) + b'\x7F'))
+    [9, 10, 13, 32, 127]
 
-
-_reference_re = re.compile(r'&(?:#[xX][0-9a-fA-F]+|#[0-9]+|\w{1,8});')
-
-def _html_parser_unescape(s):
-    """This is to avoid an issue which HTMLParser.unescape() raises
-    ValueError or OverflowError from unichr() when character reference
-    with a large integer in the attribute.
     """
-
-    def repl(match):
-        match = match.group(0)
-        name = match[1:-1]
-        if name.startswith(('#x', '#X')):
-            codepoint = int(name[2:], 16)
-        elif name.startswith('#'):
-            codepoint = int(name[1:])
-        else:
-            try:
-                codepoint = _name2codepoint[name]
-            except KeyError:
-                return match
-        if 0 <= codepoint <= 0x10ffff:
-            return _unichr(codepoint)
-        else:
-            return match
-
-    return _reference_re.sub(repl, s)
-
-
-if genshi:
-    class GenshiHTMLParserFixup(genshi.input.HTMLParser):
-
-        def handle_starttag(self, tag, attrib):
-            fixed_attrib = [(QName(name), name if value is None else value)
-                            for name, value in attrib]
-            self._enqueue(START, (QName(tag), Attrs(fixed_attrib)))
-            if tag in self._EMPTY_ELEMS:
-                self._enqueue(END, QName(tag))
-            else:
-                self._open_tags.append(tag)
-
-        def handle_charref(self, name):
-            if name.startswith(('x', 'X')):
-                codepoint = int(name[1:], 16)
-            else:
-                codepoint = int(name)
-            if 0 <= codepoint <= 0x10ffff:
-                text = _unichr(codepoint)
-            else:
-                text = '&#%s;' % name
-            self._enqueue(TEXT, text)
-
-        def handle_entityref(self, name):
-            text = None
-            try:
-                codepoint = _name2codepoint[name]
-            except KeyError:
-                pass
-            else:
-                if 0 <= codepoint <= 0x10ffff:
-                    text = _unichr(codepoint)
-            self._enqueue(TEXT, text or '&%s;' % name)
-
-        def unescape(self, s):
-            return _html_parser_unescape(s)
-
-
-    def HTML(text, encoding=None):
-        if isinstance(text, unicode):
-            f = io.StringIO(text)
-            encoding = None
-        else:
-            f = io.BytesIO(text)
-        parser = GenshiHTMLParserFixup(f, encoding=encoding)
-        return Stream(list(parser))
-
-
-    def expand_markup(stream, ctxt=None):
-        """A Genshi stream filter for expanding `genshi.Markup` events.
-
-        :deprecated: will be removed in Trac 1.5.1.
-
-        Note: Expansion may not be possible if the fragment is badly
-        formed, or partial.
-
-        """
-        for event in stream:
-            if isinstance(event[1], Markup):
-                try:
-                    for subevent in HTML(event[1]):
-                        yield subevent
-                except ParseError:
-                    yield event
-            else:
-                yield event
-else:
-    expand_markup = None
+    return data.translate(None, _invalid_control_chars)
