@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
@@ -372,7 +372,7 @@ class BrowserModule(Component):
                 rev_or_latest = rev or repos.youngest_rev
                 node = get_existing_node(req, repos, path, rev_or_latest)
             except NoSuchChangeset as e:
-                raise ResourceNotFound(e, _('Invalid changeset number'))
+                raise ResourceNotFound(e, _('Invalid changeset number')) from e
             if node:
                 try:
                     # use changeset instance to retrieve branches and tags
