@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2020 Edgewall Software
+# Copyright (C) 2003-2021 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
@@ -76,7 +76,7 @@ class IPropertyDiffRenderer(Interface):
         The rendered result can be one of the following:
          - `None`: the property change will be shown the normal way
            (''changed from `old` to `new`'')
-         - an `unicode` value: the change will be shown as textual content
+         - an `str` value: the change will be shown as textual content
          - `Markup` or `Fragment`: the change will shown as block markup
         """
 
@@ -253,7 +253,7 @@ class ChangesetModule(Component):
             new = repos.normalize_rev(new)
             old = repos.normalize_rev(old or new)
         except NoSuchChangeset as e:
-            raise ResourceNotFound(e, _("Invalid Changeset Number"))
+            raise ResourceNotFound(e, _("Invalid Changeset Number")) from e
         new_path = repos.normalize_path(new_path)
         old_path = repos.normalize_path(old_path or new_path)
         full_new_path = '/' + pathjoin(repos.reponame, new_path)
@@ -506,7 +506,7 @@ class ChangesetModule(Component):
             if old_props != new_props:
                 for k, v in sorted(old_props.items()):
                     new = old = diff = None
-                    if not k in new_props:
+                    if k not in new_props:
                         old = v  # won't be displayed, no need to render it
                     elif v != new_props[k]:
                         diff = self.render_property_diff(
@@ -520,7 +520,7 @@ class ChangesetModule(Component):
                         changed_properties.append({'name': k, 'old': old,
                                                    'new': new, 'diff': diff})
                 for k, v in sorted(new_props.items()):
-                    if not k in old_props:
+                    if k not in old_props:
                         new = browser.render_property(k, 'changeset',
                                                       new_ctx, new_props)
                         if new is not None:
@@ -681,7 +681,7 @@ class ChangesetModule(Component):
     def _render_diff(self, req, filename, repos, data):
         """Raw Unified Diff version"""
 
-        output = (line.encode('utf-8') if isinstance(line, unicode) else line
+        output = (line.encode('utf-8') if isinstance(line, str) else line
                   for line in self._iter_diff_lines(req, repos, data))
         if Chrome(self.env).use_chunked_encoding:
             length = None
@@ -852,7 +852,7 @@ class ChangesetModule(Component):
             if len(repositories) > 1:
                 filters = [
                     ('repo-' + repos.reponame,
-                     u"\xa0\xa0-\xa0" + (repos.reponame or _('(default)')))
+                     "\xa0\xa0-\xa0" + (repos.reponame or _('(default)')))
                     for repos in repositories
                     if not as_bool(repos.params.get('hidden'))
                     and repos.is_viewable(req.perm)]
@@ -953,7 +953,7 @@ class ChangesetModule(Component):
                             resource = c.resource.parent.child('source',
                                                                chg[0] or '/',
                                                                r.id)
-                            if not 'FILE_VIEW' in context.perm(resource):
+                            if 'FILE_VIEW' not in context.perm(resource):
                                 continue
                             filestats[chg[2]] += 1
                             files.append(chg[0])
@@ -975,7 +975,7 @@ class ChangesetModule(Component):
                             resource = c.resource.parent.child('source',
                                                                chg[0] or '/',
                                                                r.id)
-                            if not 'FILE_VIEW' in context.perm(resource):
+                            if 'FILE_VIEW' not in context.perm(resource):
                                 continue
                             if 0 < show_files < len(files):
                                 break
@@ -983,7 +983,7 @@ class ChangesetModule(Component):
                     files = [tag.li(tag.div(class_=mod), path or '/')
                              for path, mod in sorted(unique_files)]
                     if 0 < show_files < len(files):
-                        files = files[:show_files] + [tag.li(u'\u2026')]
+                        files = files[:show_files] + [tag.li('\u2026')]
                     markup = tag(tag.ul(files, class_="changes"), markup)
             if message:
                 markup += format_to(self.env, None,
@@ -1129,7 +1129,7 @@ class ChangesetModule(Component):
             yield ('changeset', _('Changesets'))
 
     def get_search_results(self, req, terms, filters):
-        if not 'changeset' in filters:
+        if 'changeset' not in filters:
             return
         rm = RepositoryManager(self.env)
         repositories = {repos.params['id']: repos
