@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -206,7 +206,7 @@ class HTTPException(TracBaseError, metaclass=ABCMeta):
         if args:
             self.detail = self.detail % args
         arg = '%s %s (%s)' % (self.code, self.reason, to_unicode(self.detail))
-        super(HTTPException, self).__init__(arg)
+        super().__init__(arg)
 
     @property
     def message(self):
