@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005 Matthew Good <trac@matt-good.net>
 # All rights reserved.
@@ -104,7 +104,7 @@ class RequestWithSession(Request):
     def send_response(self, code=200):
         if code < 400:
             self.session.save()
-        super(RequestWithSession, self).send_response(code)
+        super().send_response(code)
 
 
 class RequestDispatcher(Component):
@@ -295,7 +295,8 @@ class RequestDispatcher(Component):
                 self.log.warning("%s caught from %s:%d in %s: %s",
                                  e.__class__.__name__, tb[0], tb[1], tb[2],
                                  to_unicode(e) or "(no message)")
-                raise HTTPInternalServerError(TracNotImplementedError(e)) from e
+                raise HTTPInternalServerError(TracNotImplementedError(e)) \
+                      from e
             if isinstance(e, TracError):
                 raise HTTPInternalServerError(e) from e
             raise e
