@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2020 Edgewall Software
+# Copyright (C) 2005-2021 Edgewall Software
 # Copyright (C) 2005-2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -81,7 +81,7 @@ class InterTracDispatcher(Component):
                 raise TracError(
                     _("Can't view %(link)s. Resource doesn't exist or "
                       "you don't have the required permission.", link=link))
-            href = elt.attrib.get('href')
+            href = elt.attrib.get('href').unescape()
         else:
             href = req.href(link.rstrip(':'))
         req.redirect(href)
