@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2015-2020 Edgewall Software
+# Copyright (C) 2015-2021 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -37,7 +37,7 @@ class DefaultWikiChecker(Formatter):
         Formatter.__init__(self, env, context)
         self.__name = name
         self.__marks = []
-        self.__super = super(DefaultWikiChecker, self)
+        self.__super = super()
 
     def handle_match(self, fullmatch):
         rv = self.__super.handle_match(fullmatch)
